
/********************************************************************************
 *										*
 *				 M A C E K					*
 *		("Matroids Also Computed Efficiently" Kit)			*
 *										*
 * A set of tools and routines for computations with representable matroids.	*
 * Copyright (C) 2001,2002  Petr Hlineny.					*
 *										*
 *   This program is free software; you can redistribute it and/or modify	*
 *   it under the terms of the GNU General Public License as published by	*
 *   the Free Software Foundation; either version 2 of the License, or		*
 *   (at your option) any later version.					*
 *   You should have received a copy of the GNU General Public License		*
 *   along with this program; if not, write to the Free Software		*
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.	*
 * 										*
 * See the top-level "ABOUT" and "doc/ *" files for more information.		*
 *										*
 ********************************************************************************/













/**
 * A "frame" is the basic general data entity in the program.
 * Read more about a frame and its data in ../include/frame.h.
 * 
 * This source file contains declarations of command-handling functions.
 * The first part consists of definitions of the handling functions
 * (they have unified parameter-format).
 * Then there is a definition of the array comhdefs[] that holds pointers
 * to these handling functions, in association with names, comments,
 * and parameter descriptions of the commands.
 * 
 * 
 * When writing new command handles, follow these RULES:
 * 
 * make similar to existing handles.....
 * 
 * getting parameters.....
 * 	- input matrices are already copied, frames are not(!!!),
 * 	- output should be copied here (ready for putting into the frame tree),
 * 	  output frames must not have parents, but they may have descendants,
 * 	- even the output matrices must be copied again, since the input ones are disposed later!
 * 	- set the names and options here for the output frames if you want...
 * 
 * 
 * 
**/





#include "macek.h"  
#include "fr.h"










/******************	Frame i/o and handling	*******************/
/******************************************************************/


//#undef CURDLEV
#define CURDLEV         4


int	ret_yes=1, ret_no=0, ret_def=2;	/* used for yes/no lists - pointers point to these variables */


/**
 * One may control output printing verbosity within commands by a command and by an option...
 * The value of frcom_printbrief is set in frame_applycommand() based on the option @prbrief,
 * and the value of frcom_verbose is adjusted here by special commands.
**/

int	frcom_verbose = 1;
int	frcom_printbrief = 0;

int	frcom_verbose_out(void) {
	return frcom_verbose+frcom_printbrief;
}

int	COMFUNCTION(frcom_setverbose) {
	int	adj = inum[0];
	frcom_verbose += adj;
	return 0;	/* no return values */
}



/**
 * Here we defined frame-, matrix- and tree-printing functions, simple text printing, and setting a new name.
 * They do not change the frame tree.
**/

int	COMFUNCTION(frcom_prframe) {
	framestruc	**flin = (framestruc**)inls[0];
	framestruc	**x;
	
	if (flin) for (x=flin; *x; x++) {
		if (frcom_verbose_out()>0)  FROUTPUT(*x,"    ~\t");
		else  FROUTPUTS(*x,"    ~\t");
	}
	return 0;	/* no return values */
}

int	COMFUNCTION(frcom_print) {
	framestruc	**flin = (framestruc**)inls[0];
	framestruc	**x;
	ematrix		*em;
	
	if (flin) for (x=flin; *x; x++) {
		if (frcom_verbose_out()>0)  OUTPUT("Matrix of the frame %p [%s] in %s: \"%s\"\n",*x,FRNAME(*x),pfield_curname(),FRCOMMENT(*x)?FRCOMMENT(*x):"");
		em = frame_extractmatrix(*x);
		if (em) {
			if (frcom_verbose_out()>0)  EMATOUTPUT(em,"    ~\t");
			else  EMATOUTPUTS(em,"    ~\t");
			dispose_ematrix(em);
		} else  SOUTPUT("\tThere is no matrix in the frame.\n");
	}
	return 0;	/* no return values */
}

int	COMFUNCTION(frcom_printmore) {
	ematrix		**elin = (ematrix**)inls[0];
	ematrix		**el;
	long		i,j, *rem;
	
	rem = MMALLOC(sizeof(long)*(elin?alist_getlength(elin)+2:2));
	for (el=elin,i=0; el?*el:0; el++,i++) {
		if (frcom_verbose_out()>0)  EMATOUTPUT(*el,"    ~\t");
		rem[i] = ematrix_printmore(*el,frcom_verbose_out());
		if (frcom_verbose_out()>=0)
		  for (j=i-1; j>=0; j--)  if (rem[j]==rem[i]) {
			OUTPUT(" ## Repeated hash value %ld - same as for matrix #%d [%s].\n",
				rem[j],(int)j,EMNAME(elin[j]));
			if (frcom_verbose_out()>1)  EMATOUTPUT(elin[j],"    ~\t\t");
			break;
		}
	}
	FREE(rem);
	return 0;	/* no return values */
}


int	COMFUNCTION(frcom_printtree) {
	framestruc	**flin = (framestruc**)inls[0];
	framestruc	**x;
	
	if (flin) for (x=flin; *x; x++) {
		if (frcom_verbose_out()>0)  OUTPUT("Printing the subtree of the frame %p [%s]:\n",*x,FRNAME(*x));
		frame_print_tree(printout,*x,frcom_verbose_out()-1,"    ~\t");
		if (frcom_verbose_out()>0)  OUTPUT("------------------------------------\n");
	}
	return 0;	/* no return values */
}

int	COMFUNCTION(frcom_printtext) {
	char	*text = instr?instr[0]:"";
	OUTPUT("%s\n\n",text);
	return 0;	/* no return values */
}


int	COMFUNCTION(frcom_setname) {
	framestruc	**flin = (framestruc**)inls[0];
	framestruc	**x;
	
	for (x=flin; x&&instr? (*x&&instr[x-flin]):0; x++) {
		frame_setname(*x,instr[x-flin]);
	}
	return 0;	/* no return values */
}


/**
 * Here we define frame and frame-tree file writing functions.
 * The filename is taken from the name of the current frame, or "noname", or
 * can be set by the instr[0] string parameter (only for the first frame!).
**/

int	COMFUNCTION(frcom_write) {
	framestruc	**flin = (framestruc**)inls[0];
	char		*dir, *fn = instr?instr[0]:NULL;
	extern char	*fopen_dirused;	/* (records the directory found in the path during open) */
	framestruc	**x;
	int		k;
	
	dir = (fn? (fn[0]&&fn[strlen(fn)-1]=='/'? fn:NULL):NULL);
	for (x=flin; x?*x:0; x++) {
		if (x==flin && fn && !dir)  frame_setname(*x,fn);
		if (frcom_verbose_out()>0)  OUTPUT("Writing the frame %p [%s]  \"%s\".\n",*x,FRNAME(*x),FRCOMMENT(*x)?FRCOMMENT(*x):"");
		if (frcom_verbose_out()>2)  FROUTPUT(*x,"    ~\t");
		else if (frcom_verbose_out()>1)  FROUTPUTS(*x,"    ~\t");
		
		k = frame_writed_one(*x,dir,FRCOMMENT(*x));
		if (k>=0)  OUTPUT("Written to \"%s / %s / %s\"; OK.\n",
				fopen_dirused,dir?dir:"",FRNAME(*x));
		else  OUTPUT("Write \"%s\" ERROR\n",FRNAME(*x));
	}
	return 0;	/* no return values */
}

int	COMFUNCTION(frcom_writetree) {
	framestruc	**flin = (framestruc**)inls[0];
	char		*dir, *fn = instr?instr[0]:NULL;
	extern char	*fopen_dirused;	/* (records the directory found in the path during open) */
	framestruc	**x;
	int		k;
	char		*buf;
	
	dir = (fn? (fn[0]&&fn[strlen(fn)-1]=='/'? fn:NULL):NULL);
	buf = MMALLOC(50400);
	if (flin)  for (x=flin; *x; x++) {
		if (x==flin && fn && !dir)  frame_setname(*x,fn);
		if (frcom_verbose_out()>0)  OUTPUT("Writing the subtree of the frame %p [%s]  \"%s\".\n",
				*x,FRNAME(*x),FRCOMMENT(*x)?FRCOMMENT(*x):"");
		if (frcom_verbose_out()>3)  frame_print_tree(printout,*x,frcom_verbose_out()-4,"    ~\t");
		else if (frcom_verbose_out()>2)  FROUTPUT(*x,"    ~\t");
		else if (frcom_verbose_out()>1)  FROUTPUTS(*x,"    ~\t");
		
		strcpy(buf,"\nFrame tree:\n");
		frame_printtree_to(buf+strlen(buf),50000,*x);  buf[50000] = 0;
		if (strlen(buf)>50000-40)  sprintf(buf+strlen(buf),"\n\t.......(possibly unfinished printout).......\n");
		k = frame_writed_tree(*x,dir,buf);
		if (k>=0)  OUTPUT("Written to \"%s / %s / %s\"; OK.\n",
				fopen_dirused,dir?dir:"",FRNAME(*x));
		else  OUTPUT("Write \"%s\" ERROR\n",FRNAME(*x));
	}
	FREE(buf);
	return 0;	/* no return values */
}


/**
 * This reads an additional input frame (with a possible subtree) from the given
 * string parameter, and arranges the new frame within the frame tree.
 * Commands in the new frame are processed before returning.
**/

int	COMFUNCTION(frcom_read) {
	char		*rdnm = instr?instr[0]:"no";
	framestruc	*in;
	int		r;
	
	in = frame_doinput(rdnm);
	r = in? frame_processcommands(in):-111;
	if (r<0)  {USERERROR("An error %d occured when reading \"%s\", cancelling",r,rdnm);}
	else  outls[0] = alist_append(outls[0],in);
	return FRES_STOREFR|FRES_STOREREM;	/* of type FCRET_ONE */
}


/**
 * Here we define function appending given (additional) input string to the given
 * frame(s), appending commands/options, and openning the matrix again for input.
 * Nothing is returned, the frame itself is modified.
**/

int	COMFUNCTION(frcom_append) {
	framestruc	**flin = (framestruc**)inls[0];
	char		*apstr = instr?instr[0]:NULL;
	framestruc	**x, *f;
	
	for (x=flin; x&&apstr?*x:0; x++) {
		OUTPUT("Appending \'%.20s\' to frame \"%s\"...\n",apstr,FRNAME(*x));
		f = frame_doinput_ap(apstr,*x);
		if (f!=*x)  {PROGERROR("The frame address has changed after an append command %p!=%p!",*x,f);}
	}
	return 0;	/* no return values */
}


/**
 * This is a general manipulating function for the frame tree.
 * It can move, copy or delete frames depending on the input and output parameters.
 * (The functionality is actually defined by the parameter processing in frame_applycommand(),
 * this function only copies the given frames to the output list.)
 * The next one is a similar function just for matrices (copied to new frames).
**/

int	COMFUNCTION(frcom_move) {
	framestruc	**flin = (framestruc**)inls[0];
	framestruc	**x;
	
	outls[0] = new_alist(10);
	if (flin) for (x=flin; *x; x++) {
		outls[0] = alist_append(outls[0],frame_copy_all(*x));
		//********** renaming copied frames........ ???
	}
	return FRES_STOREFR|FRES_DELETEMARK|FRES_STOREREM;	/* of type FCRET_PARTLIST */
}

int	COMFUNCTION(frcom_mmove) {
	ematrix		**elin = (ematrix**)inls[0];
	ematrix		**el, *em;
	
	outls[0] = new_alist(10);
	for (el=elin; *el; el++) {
		em = ematrix_copy(*el);  EMSETNAME(em,EMNAME(*el));
		outls[0] = alist_append(outls[0],em);
	}
	return FRES_STOREMX|FRES_STOREMXNM|FRES_DELETEMARK|FRES_STOREREM;/* of type FCRET_PARTLIST */
}


/**
 * This is a general manipulating function for subtrees in the frame tree.
 * All frames in the subtrees of the given frames are copied to one list
 * which is stored on the output.
**/

int	COMFUNCTION(frcom_flatten) {
	framestruc	**flin = (framestruc**)inls[0];
	framestruc	**x,**y, **tr;
	
	outls[0] = new_alist(10);
	if (flin) for (x=flin; *x; x++) {
		tr = frame_gettree_dep(*x);
		for (y=tr; y?*y:0; y++)
			outls[0] = alist_append(outls[0],frame_copy_all(*y));
	}
	return FRES_STOREFR|FRES_DELETEMARK|FRES_STOREREM;	/* of type FCRET_LIST */
}












/******************	Basic matrix commands	*******************/
/******************************************************************/



/**
 * This function is used as a general finishing to single-matrix manipulating functions.
 * Depending on rettype, it prints or stores the resulting matrix em.
**/

int	frcom_decide_printmx(ematrix *em, void **outls[], int rettype) {
	
	if (FCRET_PRINTOWN(rettype)) {	/* printed-only version of the command */
		if (frcom_verbose_out()>0)  EMATOUTPUT(em,"    ~\t");
		else  EMATOUTPUTS(em,"    ~\t");
		dispose_ematrix(em);
	} else {
		outls[0] = alist_append(outls[0],em);
	}
	return (FCRET_PRINTOWN(rettype)?0: FRES_STOREMX|FRES_STOREREM);
}


/**
 * This is a handle for matrix pivoting; "!pivot row col matrix".
 * Only one input matrix is taken, it is copied and then the pivot is applied.
 * See frcom_decide_printmx() for final finishing...
**/

int	COMFUNCTION(frcom_pivot) {
	ematrix		*ein = inls[0][0];
	int		r = inum[0], c = inum[1];
	
	if (!ein)  return 0;
	if (r>0 && r<=ROWSM(ein) && c>0 && c<=COLSM(ein) && SIGNM(ein,r-1,c-1)!=0) {
		ein = ematrix_copy(ein);
		ematrix_pivot(ein,r-1,c-1);
	} else {
		USERERROR("Invalid pivot on an entry %d,%d.",r,c);  EMATDEBUG(0,ein,"!\t");
		return -1;
	}
	return frcom_decide_printmx(ein,outls,rettype);	/* of type FCRET_REPLMX1 */
}


/**
 * These functions delete (contract) the line given by its id (not row or column!).
 * The result is pronted or stored by frcom_decide_printmx().
**/

int	COMFUNCTION(frcom_delete) {
	ematrix		*ein = inls[0][0];
	int		id = inum[0];
	
	if (!ein)  return 0;
	ein = ematrix_removematid_del(ein,id);
	return frcom_decide_printmx(ein,outls,rettype);	/* of type FCRET_REPLMX1 */
}

int	COMFUNCTION(frcom_contract) {
	ematrix		*ein = inls[0][0];
	int		id = inum[0];
	
	if (!ein)  return 0;
	ein = ematrix_removematid_contr(ein,id);
	return frcom_decide_printmx(ein,outls,rettype);	/* of type FCRET_REPLMX1 */
}


/**
 * These functions delete (contract) all elements of the given matrix one by one,
 * creating an output list of minors...
**/

int	COMFUNCTION(frcom_deleteeach) {
	ematrix		**elin = (ematrix**)inls[0];
	ematrix		**el, *ee;
	int		i,id;
	
	for (el=elin; el?*el:0; el++)	/* all given input matrices are processed here */
		for (i=0; i<ROWSM(*el)+COLSM(*el); i++) {
			id = i<ROWSM(*el)? ROWSID(*el,i): COLSID(*el,i-ROWSM(*el));
			ee = ematrix_removematid_del(*el,id);
			if (EMNAME(ee))  FREE(EMNAME(ee));  EMSETNAME(ee,EMNAME(*el));
			outls[0] = alist_append(outls[0],ee);
		}
	return FRES_STOREMX|FRES_STOREREM|FRES_DELETEMARK|FRES_STOREMXNM;	/* of type FCRET_LISTMX */
}

int	COMFUNCTION(frcom_contracteach) {
	ematrix		**elin = (ematrix**)inls[0];
	ematrix		**el, *ee;
	int		i,id;
	
	for (el=elin; el?*el:0; el++)	/* all given input matrices are processed here */
		for (i=0; i<ROWSM(*el)+COLSM(*el); i++) {
			id = i<ROWSM(*el)? ROWSID(*el,i): COLSID(*el,i-ROWSM(*el));
			ee = ematrix_removematid_contr(*el,id);
			if (EMNAME(ee))  FREE(EMNAME(ee));  EMSETNAME(ee,EMNAME(*el));
			outls[0] = alist_append(outls[0],ee);
		}
	return FRES_STOREMX|FRES_STOREREM|FRES_DELETEMARK|FRES_STOREMXNM;	/* of type FCRET_LISTMX */
}


/**
 * This function simply transposes the given matrix(ces) in the list.
**/

int	COMFUNCTION(frcom_dual) {
	ematrix		**elin = (ematrix**)inls[0];
	ematrix		**el, *ee;
	
	for (el=elin; el?*el:0; el++) {		/* all given input matrices are transposed here */
		ee = ematrix_copydual(*el);
		outls[0] = alist_append(outls[0],ee);	/* the output list is then stored back to the frames */
	}
	return FRES_STOREMX|FRES_STOREREM;	/* of type FCRET_REPLMXL */
}


/**
 * This function is for switching the current pfield to the new by given name.
 * 
**/

int	COMFUNCTION(frcom_pfield) {
	char	*pftx = instr?instr[0]:"";
	int	r;
	
	r = pfield_switchto_ext(0,pftx);
	if (r<0)  {USERERROR("Failed to switch pfield to \"%s\", now in \"%s\"\n",pftx,pfield_curname());}
	else  OUTPUT("Switched arithmetics to a new pfield \"%s\"\n",pfield_curname());
	return 0;	/* no return values */
}

/**
 * This function is provided for importing matrices of the given frames by the
 * given pf translation.
 * The imported copies of matrices are returned back for storing in the frames.
**/

int	COMFUNCTION(frcom_import) {
	char		*trtx = instr?instr[0]:"";
	framestruc	**flin = (framestruc**)inls[0];
	framestruc	**x;
	ematrix		*ee;
	
	if (pfield_switchtrans(trtx)<0) {	/* first must select the given pf translation */
		USERERROR("Setting pfield translation \"%s\" failed!",trtx);
		return -1;
	}
	if (flin) for (x=flin; *x; x++) {	/* imports copies of matrices of all given frames */
		pfield_translation(&FRVALUEEXP(*x),&FRVALUESIG(*x));
		ee = frame_extractmatrix(*x);
		if (!ee)  continue;
		ematrix_import(ee);	/* (calls pfield_translation() for all entries inside) */
		outls[0] = alist_append(outls[0],ee);	/* the output list is then stored back to the frames */
		FRPFINDEX(*x) = pfield_curindex();
	}
	return FRES_STOREMX|FRES_STOREREM;	/* of type FCRET_REPLMXL */
}






/**
 * 
 * 
 * sample...
**/

int	COMFUNCTION(frcom_) {
	ematrix		*ein = inls[0][0];
	
	outls[0] = alist_append(outls[0],ein);
	return FRES_STOREMX;	/* of type FCRET_REPLMX1 */
}













/******************	Structural matrix tests	*******************/
/******************************************************************/



/**
 * This function is used as a general finishing to frame-list returning functions.
 * Depending on rettype, it ...
**/

int	frcom_decide_outlist(void **outls[], int outnum[], int rettype) {
	
	if (outnum[0]<0)	/* must give the default list value to process the yes/no list */
		outnum[0] = ((FCRET_ISYESNOY(rettype) || rettype==FCRET_YESNOPRINT)? ret_yes:ret_no);
	
	if (FCRET_ISYESNO(rettype) && !FCRET_PRINTREQ(rettype)) {    /* of type FCRET_YESNO??? */
		if ((rettype)==FCRET_YESNOFR || (rettype)==FCRET_NOYESFR)
			return FRES_STOREFR|FRES_DELETEMARK|FRES_STOREREM;
		else  return FRES_STOREFR|FRES_STOREREM;
	} else  return 0;		/* of type FCRET_PRINTED? */
}


/**
 * This function is handling fans in the given matrices (frames).
 * If inum[] param is given, then we ask for a fan of size inum[0], otherwise for
 * the maximal fan.
 * The question about a fan of size inum[0] may generate a yes/no list.
**/

int	COMFUNCTION(frcom_hasfan) {
	framestruc	**flin = (framestruc**)inls[0];
	int		fan = ((inumn>0&&inum?(inum[0]!=0):0)? inum[0]: -1);
	framestruc	**x;
	int		i,r;
	
	for (x=flin,i=0; x?*x:0; x++,i++) {
		if (FCRET_PRINTOWN(rettype) && frcom_verbose_out()>1)
			struct_hasfan_printit(printout,"    ~\t");
		if (fan<0)  r = struct_hasfan_max(FRMATRIX(*x));
		else  r = struct_hasfan_all(FRMATRIX(*x),fan);
		
		if (FCRET_ISYESNOPR(rettype))
			outls[0] = alist_append(outls[0],&(r>0?ret_yes:ret_no));
		if (FCRET_PRINTOWN(rettype))
			OUTPUT("The #%d matroid %p [%s] has%s fan (>=3) of size %d.\n",
					i+1,*x,FRNAME(*x),r>0?"":" NO",fan>0?fan:r);
	}
	return frcom_decide_outlist(outls,outnum,rettype);
}


/**
 * This is a handling function for matroid minors.
 * It looks whether the given frame(s) has a minor in the given minor-list inls[1].
 * Moreover, if the minor-list contains an equal matrix, during printing(!) command only,
 * this is reported and the minor removed.
**/

int	COMFUNCTION(frcom_minorlist) {
	framestruc	**flin = (framestruc**)inls[0];
	ematrix		**elmin = (ematrix**)inls[1];
	ematrix		**el, **y;
	framestruc	**x;
	int		i,r;
	
	for (x=flin,i=0; x?*x:0; x++,i++) {
		if (!FRMATRIX(*x))  continue;
		el = elmin;
		if (FCRET_PRINTOWN(rettype))	/* looking if there is one equal in the minors */
		  for (y=elmin; y?*y:0; y++)
			if (ematrix_isequal(*y,FRMATRIX(*x))) {
				OUTPUT("The #%d matroid %p [%s] is +EQUAL+ to minor #%d in the minor list, removing.\n",
					i+1,*x,FRNAME(*x),(int)(y-elmin)+1);
				el = alist_copy(y);
				el = alist_getstart(alist_delete(el));
				break;	/* an equal matrix is removed from the minor-list for printing */
			}
		if (FCRET_PRINTOWN(rettype) && frcom_verbose_out()>1) {
			if (frcom_verbose_out()>2)
				struct_hasminor_printall(printout,"    ~\t\t");
			else  struct_hasminor_printit(printout,"    ~\t\t");
		}
		r = !el?0: struct_hasminorlist(FRMATRIX(*x),el);
		
		if (FCRET_ISYESNOPR(rettype))
			outls[0] = alist_append(outls[0],&(r?ret_yes:ret_no));
		if (FCRET_PRINTOWN(rettype) && (r||el==elmin))
			OUTPUT("%sThe #%d matroid %p [%s] %s minor in the %slist [%.7s %.7s%s].\n",el!=elmin?"  ":"",
				i+1,*x,FRNAME(*x),r?"+HAS+":"has -NO-",el!=elmin?"reduced ":"given ",
				(el?el[0]:0)?EMNAME(el[0]):"",(el?el[0]&&el[1]:0)?EMNAME(el[1]):"",(el?el[0]&&el[1]&&el[2]:0)?" ..":"");
		if (el!=elmin)  alist_free(el);
	}
	return frcom_decide_outlist(outls,outnum,rettype);
}


/**
 * This is a handling function for testing the input frames as tight majors of the next
 * given minor list.
**/

int	COMFUNCTION(frcom_tmajorlist) {
	framestruc	**flin = (framestruc**)inls[0];
	ematrix		**elmin = (ematrix**)inls[1];
	framestruc	**x;
	int		i,r,rr, id;
	
	for (x=flin,i=0; x?*x:0; x++,i++) {
		rr = struct_hasminorlist(FRMATRIX(*x),elmin);
		if (!rr)  OUTPUT("The #%d matroid %p [%s] has !NO! minor in the given list.\n",i+1,*x,FRNAME(*x));
		r = struct_hasdelcontr_wh(FRMATRIX(*x),elmin,&id);
		
		if (FCRET_ISYESNOPR(rettype))
			outls[0] = alist_append(outls[0],&(r?ret_yes:ret_no));
		if (FCRET_PRINTOWN(rettype))
			OUTPUT("%sThe #%d matroid %p [%s] %s a tight major (%s%d) of the given list [%.7s %.7s%s].\n",
				rr?"":"  ",i+1,*x,FRNAME(*x),!r?"+IS+":"is -NOT-",r?"rem ":"",id,
				(elmin?elmin[0]:0)?EMNAME(elmin[0]):"",(elmin?elmin[0]&&elmin[1]:0)?EMNAME(elmin[1]):"",(elmin?elmin[0]&&elmin[1]&&elmin[2]:0)?" ..":"");
	}
	return frcom_decide_outlist(outls,outnum,rettype);
}


/**
 * This is a handling function for testing proper matrix representation over the pfield.
 * (If some matrices are not properly represented, use option @nopfcheck...)
**/

int	COMFUNCTION(frcom_pfreprlist) {
	framestruc	**flin = (framestruc**)inls[0];
	framestruc	**x;
	int		i,r;
	
	for (x=flin,i=0; x?*x:0; x++,i++) {
		r = ematrix_inpfield(FRMATRIX(*x))>=0;
		if (!r && FCRET_PRINTOWN(rettype) && frcom_verbose_out()>1)
			ematrix_inpfield_printed(FRMATRIX(*x));
		
		if (FCRET_ISYESNOPR(rettype))
			outls[0] = alist_append(outls[0],&(r?ret_yes:ret_no));
		if (FCRET_PRINTOWN(rettype))
			OUTPUT("The #%d matroid %p [%s] %s properly represented over %s.\n",
				i+1,*x,FRNAME(*x),r?"+IS+":"is -NOT-",pfield_curname());
	}
	return frcom_decide_outlist(outls,outnum,rettype);
}


/**
 * This is a handling function for testing branchwidth3.
 * So far it works only for 3-connected matroids!!!
 * 
**/

int	COMFUNCTION(frcom_bwidth3test) {
	framestruc	**flin = (framestruc**)inls[0];
	framestruc	**x;
	int		i,b;
	
	for (x=flin,i=0; x?*x:0; x++,i++) {
		b = struct_hasbwidth3_ext(1,FRMATRIX(*x));
		
		if (FCRET_ISYESNOPR(rettype))
			outls[0] = alist_append(outls[0],&(b?ret_yes:ret_no));
		if (FCRET_PRINTOWN(rettype))
			OUTPUT("The #%d matroid %p [%s] %s brach-width <=3.\n",i+1,*x,FRNAME(*x),b?"+HAS+":"has -NOT-");
	}
	return frcom_decide_outlist(outls,outnum,rettype);
}


/**
 * This function tests equivalence of the matroids in the first list to some members
 * of the second list.
**/

int	COMFUNCTION(frcom_equivalent) {
	framestruc	**flin = (framestruc**)inls[0];
	ematrix		**eleq = (ematrix**)inls[1];
	ematrix		**y, *ee;
	framestruc	**x;
	int		i,r,q;
	
	if (flin) for (x=flin,i=0; *x; x++,i++) {
		if (!FRMATRIX(*x))  continue;
		for (r=q=0,y=eleq,ee=NULL; y?*y:0; y++) {
					/* looking if there is one equal in the minors */
			if (FCRET_PRINTOWN(rettype)) if (ematrix_isequal(*y,FRMATRIX(*x))) {
				if (frcom_verbose_out()>1)  EMATOUTPUTS(*y,"    ~\t\t");
				OUTPUT("The #%d matroid %p [%s] is +EQUAL+ to the #%d matroid [%s] in the second list.\n",
						i+1,*x,FRNAME(*x),(int)(y-eleq)+1,EMNAME(*y));
				q = 1;  continue;
			}
			if (FCRET_PRINTOWN(rettype) && frcom_verbose_out()>1)
				struct_hasminor_printit(printout,"    ~\t\t");
			if (struct_isequivalent(*y,FRMATRIX(*x))) {
				r = 1;  ee = *y; }
		}
		if (FCRET_ISYESNOPR(rettype))
			outls[0] = alist_append(outls[0],&(r?ret_yes:ret_no));
		if (FCRET_PRINTOWN(rettype) && (r||!q))
			OUTPUT("%sThe #%d matroid %p [%s] %s equivalent to [%s] in the second list [%.7s %.7s%s].\n",
				q?"  ":"",i+1,*x,FRNAME(*x),r?"+IS+":"is -NOT-",ee?EMNAME(ee):"",
				(eleq?eleq[0]:0)?EMNAME(eleq[0]):"",(eleq?eleq[0]&&eleq[1]:0)?EMNAME(eleq[1]):"",(eleq?eleq[0]&&eleq[1]&&eleq[2]:0)?" ..":"");
	}
	return frcom_decide_outlist(outls,outnum,rettype);
}


/**
 * This special handling function is provided for "cleaning" lists of matroids from
 * isomorphic (non-equivalent) pairs.
 * The idea is that the first given list is that one to clean, and the second given list
 * contains alternative representations of the matroids from the first list (in one-to-one
 * correspondence).
 * Only those matroids of the first list are kept that have no equivalent one further
 * down in any of the two lists.
**/

int	COMFUNCTION(frcom_makeunique) {
	framestruc	**flin = (framestruc**)inls[0];
	ematrix		**ealt = (ematrix**)inls[1];
	framestruc	**x;
	ematrix		*ee, **y,**z,**le;
	int		i,j, r,er;
	
	if (!flin || !ealt)  {USERERROR("Both lists must be given here!"); return 0;}
	if (alist_getlength(flin)!=alist_getlength(ealt))  {USERERROR("The given lists are supposed to have equal length!"); return 0;}
	er = 0;  le = NULL;
			/* preparing the corresponding pairs into one list */
	for (x=flin,i=0; x?*x:0; x++,i++) {
		ee = FRMATRIX(*x);  if (!ee)  {PROGERROR("Matrix must be given here!"); return 0;}
		if (ROWSM(ee)+COLSM(ee)<12) if (ematrix_getmhash(ee)!=ematrix_getmhash(ealt[i]))
			if (!er)  {er = 1; USERERROR("The given lists are supposed to contain pairs of isomorphic matroids!");}
		le = alist_append(le,ee);  le = alist_append(le,ealt[i]);
	}
			/* processing the prepared list - first of each pair against all remaining pairs */
	for (y=le,i=0; y?(y[0]&&y[1]):0; y+=2,i++) {
		ee = *y;
		for (z=y+2,r=0; *z && !r; z++)
			r = struct_isequivalent(ee,*z)? 1:r;
		if (FCRET_ISYESNOPR(rettype))
			outls[0] = alist_append(outls[0],&(r?ret_no:ret_yes));
		if (FCRET_PRINTOWN(rettype)) {
			j = (int)(z-y-1)/2+i;
			if (r && frcom_verbose_out()>1)  EMATOUTPUTS(ee,"    ~\t\t");
			if (r && frcom_verbose_out()>1)  EMATOUTPUTS(z[-1],"    ~\t\t>");
			if (r)  OUTPUT("The #%d matroid %p [%s] is equivalent to the #%d matroid [%s], REMOVED.\n",
					i+1,flin[i],FRNAME(flin[i]),j+1,FRNAME(flin[j]));
			if (!r && frcom_verbose_out()>0)  OUTPUT("The #%d matroid %p [%s] is unique in the lists.\n",
					i+1,*y,FRNAME(flin[i]));
		}
	}
	alist_free(le);
	return frcom_decide_outlist(outls,outnum,rettype);
}


/**
 * This function is looking for matroids of given hash value in the list.
 * (A matroid hash value is computed from the basis-structure of the matrix,
 * and hence it is matroid-invariant and may be used to informally compare
 * two matroids over different pfields.)
 * The function may generate a yes/no list for filtering.
**/

int	COMFUNCTION(frcom_matroidhash) {
	framestruc	**flin = (framestruc**)inls[0];
	long		hash = ((inumn>0&&inum?(inum[0]!=0):0)? inum[0]: -1);
	framestruc	**x;
	int		i,r;	long	hh;
	
	for (x=flin,i=0; x?*x:0; x++,i++) {
		hh = ematrix_getmhash(FRMATRIX(*x));
		if (hh!=((int)hh))  {PROGERROR("Hash value probably too big for integer parameter.");}
		r = ((int)hh==(int)hash? 1:-1);
		
		if (FCRET_ISYESNOPR(rettype))
			outls[0] = alist_append(outls[0],&(r>0?ret_yes:ret_no));
		if (FCRET_PRINTOWN(rettype)) {
			if (frcom_verbose_out()>1)  EMATOUTPUTS(FRMATRIX(*x),"    ~\t\t");
			OUTPUT("The #%d matroid %p [%s] %s matroid hash-value \t%ld %s %ld.\n",
					i+1,*x,FRNAME(*x),r>0?" +HAS+ ":"has NOT",hh,r>0?"==":"!=",hash);
		}
	}
	return frcom_decide_outlist(outls,outnum,rettype);
}













/******************	Matrix constructions	*******************/
/******************************************************************/



/**
 * This is a handle for matrix automorphisms - the automorphisms are printed on the screen.
 * Nothing is returned.
**/

int	COMFUNCTION(frcom_automorph) {
	ematrix		*ein = inls[0][0];
	emxaut		**al, **x;
	
	if (!ein)  return 0;
	al = struct_automorphisms(ein);
	OUTPUT("\tFound %d (scaled) automorphisms of the matrix %p [%s].\n",alist_getlength(al),ein,EMNAME(ein));
	if (al && frcom_verbose_out()>0)  for (x=al; *x; x++)
		emxaut_print(printout,*x,"    ~\t  ");
	struct_automorphisms_recycle(ein,al);
	return 0;	/* of type FCRET_PRINTED */
}



/**
 * This function generates 3-connected extensions of the matrix(ces) in the frame(s) flin.
 * The string parameter what tells which extensions should be generated - r for a row,
 * c for a column, b for both.
 * One may give a sequence like "rcbb..." to generate several steps at once.
 * Or, one may give, instead of what, two number parameters rrow,rcol to generate all
 * extensions up to the dimensions rrow,rcol of the resulting matrix.
 * 
 * If using any form of multi-step extension, the input may contain only one matrix.
 * The result is returned as a list of all generated matrices (not containing input ones),
 * or only those of dimensions rrow,rcol if this is requested.
 * Read ../gener/gener.c.
 * 
 * IMPORTANT: The generating process uses internal options @bsize and @signature
 * that are initialized at the first step of generating.
 * They should never be set by hand, as doing so may damage the whole process.
 * All steps of generating extensions must be 3-connected, including the starting one!
 * Read carefully the theory in ../include/gener.h.
 * 
**/

int	COMFUNCTION(frcom_extend) {
	framestruc	**flin = (framestruc**)inls[0];
	char		*what = instr?instr[0]:"";
	int		rrow = inumn>=2?inum[0]:0,  rcol = inumn>=2?inum[1]:0;
	framestruc	**x,**y, **out;
	int	i, ext=0, cox=0;
	
	if (rrow>0 && rcol>0)  what = "zzzzzzzzzzzz";
	out = flin;
	for (; out && what[0]; what++) {
		if (out!=flin && flin[1])  {USERERROR("Cannot correctly do multiple-steps with more than one input matrix!");}
		if (what[0]!='r' && what[0]!='c' && what[0]!='b' && what[0]!='z')  {USERERROR("Wrong extension description string \"%s\".",what);}
		y = NULL;
		ext = (what[0]=='c' || what[0]=='b');	/* (extend column or both) */
		cox = (what[0]=='r' || what[0]=='b');	/* (extend row or both) */
		
		for (x=out,i=0; x?*x:0; x++,i++) {
			if (what[0]=='z') {	/* special - extending all to given size rrow x rcol */
				ext = (COLSM(FRMATRIX(*x))<rcol);
				cox = (ROWSM(FRMATRIX(*x))<rrow);
				if (!ext && !cox)  outls[0] = alist_append(outls[0],frame_copy_all(*x));
			}
				/* must (temporarily) give the parent for inheriting options! */
			if (out!=flin)  frame_setson(*x,flin[0]);
			if (ext)  y = alist_applist(y,gener_extframe(*x,1));
			if (cox)  y = alist_applist(y,gener_extframe(*x,0));
			//********** later use the ctrl-parameter of generating ????
			
			if (out!=flin)  frame_setson(*x,NULL);
		}
				/* dispose intermediate list if generating to size, or store it if generating by symbols */
		if (out!=flin && what[0]=='z')  dispose_alist_frams(out);
		if (out!=flin && what[0]!='z')  outls[0] = alist_applist(outls[0],out);
		out = y;	/* (we cannot append the list y here since appending would destroy it!!!) */
	}
	
	if (out!=flin)  outls[0] = alist_applist(outls[0],out);
	if (!outls[0])  outls[0] = new_alist(4);
	return FRES_DELETEMARK|FRES_STOREFR|FRES_STOREREM;
}



















/******************	Command-flow Control	*******************/
/******************************************************************/



/**
 * This function requests to restart command processing in the whole tree.
 * The commands which were already processed are not repeated, but new commands
 * later added with '!append' get chance...
**/

int	COMFUNCTION(frcom_restart) {
	
	OUTPUT("\tRestarting command processing in the whole tree...\n");
	if (frcom_verbose_out()>0)  OUTPUT("\t (Commands already processed are not repeated.)\n");
	return FRES_RESTART;
}


/**
 * These command handles are for skipping and exitting from command processing.
 * 
**/

int	COMFUNCTION(frcom_comexit) {
	int	num = inumn>=1?inum[0]:0;
	
	if (frcom_verbose_out()>0)  OUTPUT("\tExit %d from the program command processing.\n",num);
	if (num<0)  DEBUG(CURDLEV-1,"Negative return value %d is returned as %d !\n",num,FRES_PROCSTRIP2(num));
	frame_skipcommands = -1;
	return FRES_PROCSTRIP2(num)|FRES_SKIPCOM;
}

int	COMFUNCTION(frcom_comskip) {
	int	num = inumn>=1?inum[0]:0;
	
	if (frcom_verbose_out()>0)  OUTPUT("\tSkipping the next %d commands in processing.\n",num);
	frame_skipcommands = num>0?num:0;
	return FRES_SKIPCOM;
}


/**
 * This is a command testing length of the given frame list.
 * If the test is true, then the next command is executed, otherwise it is skipped.
 * 
**/

int	COMFUNCTION(frcom_iflist) {
	framestruc	**flin = (framestruc**)inls[0];
	int		len = inumn>=1?inum[0]:0;
	char		*what = instr?instr[0]:"";
	int		r,ll;
	
	ll = alist_getlength(flin);
	r = 0;
	if (what[0]=='!' || strcmp(what,"<>")==0) {
		r = (len!=ll);
	} else if (what[0]=='=' && strcmp(what,"=!")!=0) {
		r = (len==ll);
	} else if (what[0]=='<') {
		r = what[1]=='='? (len<=ll): (len<ll);
	} else if (what[0]=='>') {
		r = what[1]=='='? (len>=ll): (len>ll);
	} else {
		USERERROR("Invalid comparing \"%s\" in !iflist.",what);
	}
	if (frcom_verbose_out()>0)  OUTPUT("\tIf-list length test %d %s %d, result %s\n.",len,what,ll,r?"YES":"NO");
	frame_skipcommands = r?0:1;
	return (r?0:FRES_SKIPCOM)|FRES_STOREREM;
}






















int	COMFUNCTION(frcom_test) {
	//ematrix	*ein = inls[0][0];	ematrix		**exl;
	framestruc	**flin = (framestruc**)inls[0];
	framestruc	**x,**y=NULL;
	int	i;
	
	//exl = gener_matextens_ext(2,ein);
	//dispose_alist_mats(exl);
	
	if (flin) for (x=flin,i=0; *x; x++,i++) {
		y = gener_extframe_ext(*x,0,0);
	}
	
	outls[0] = (void**)y;
	//return 0;
	//return FRES_STOREMX;
	return FRES_DELETEMARK|FRES_STOREFR;
}























/******************	Definition table for command handles	*******************/
/**********************************************************************************/


#undef CURDLEV
#define CURDLEV         4


/**
 * Extra third-party handles may be included here.......
 * 
**/

#define	FR_COMSEXTRA	0
#include "frcoms-more.inc"
#undef	FR_COMSEXTRA



/**
 * Here is the definition of the command handling array...
 * Read more about input/output parameter descriptions in ../include/frame.h,
 * including the definitions of the FCPAR_* and FCRET_* macros.
 * 
 * The definition of the whole structure commhandles is in fr.h .
 * In particular, the second field is the pattern for getting new frame name.
 * It gets the output name from the first input frame, and
 * values of index and command parameters (use %i$ conversion to change the order):
 	snprintf(buf,490,cmf.fnformat, nmret,i,
                  OPTPARAM(com,0),OPTPARAM(com,1),OPTPARAM(com,2),OPTPARAM(com,3));
 * (No second 'i' after nmret is given when printing name for FCRET_REPLMX[1L] !)
 * The default pattern FCDEFNFORMAT is in fr.h.
 * 
**/


commhandles	comhdefs[] = {
	
	{
	
		/*********	Frame i/o and handling	*********/
		/************************************************/
		
		"pr",	NULL,	{NULL},	NULL,
		"Use: \"!pr frame-list\"; \tprints the given frames.",
		&frcom_prframe,
		1, { FCPAR_LISTFR0 },
		FCRET_PRINTED
	},{
		"print",	NULL,	{NULL},	NULL,
		"Use: \"!print frame-list\"; \tprints the matrices of the given frames.",
		&frcom_print,
		1, { FCPAR_LISTFRM0 },
		FCRET_PRINTED
	},{
		"printmore",	NULL,	{NULL},	NULL,
		"Use: \"!printmore mat-list\"; \tprints info about the matroids in the frames.",
		&frcom_printmore,
		1, { FCPAR_LISTMX0 },
		FCRET_PRINTED
	},{
		"prtree",	NULL,	{ "(T)", NULL},	NULL,
		"Use: \"!prtree frame-list\"; \tprints the frame subtree (not matrices).",
		&frcom_printtree,
		1, { FCPAR_LISTFR0 },
		FCRET_PRINTED
	},{
		"prtext",	NULL,	{NULL},	NULL,
		"Use: \"!prtext text\"; \tprints the given text.",
		&frcom_printtext,
		1, { FCPAR_STRING },
		FCRET_PRINTED
	},{
		"verbose",	NULL,	{ "1", NULL},	NULL,
		"Use: \"!verbose [+-n]\"; \tadjusts the current output verbosity (default up).",
		&frcom_setverbose,
		1, { FCPAR_INTEGER },
		FCRET_NOTHING
	},{
		"quiet",	NULL,	{ "-1", NULL},	NULL,
		"Use: \"!quiet [-+n]\"; \tadjusts the current output verbosity (default down).",
		&frcom_setverbose,
		1, { FCPAR_INTEGER },
		FCRET_NOTHING
	},{
		"setname",	NULL,	{NULL},	NULL,
		"Use: \"!setname name frame\"; \tsets the given name to the frame.",
		&frcom_setname,
		2, { FCPAR_STRING, FCPAR_ONEFR },
		FCRET_NOTHING
	},{
		"write",	NULL,	{NULL},	NULL,
		"Use: \"!write frame(s)\"; \twrites the frame to a file (with matrices).",
		&frcom_write,
		1, { FCPAR_LISTFR },
		FCRET_PRINTED
	},{
		"writeto",	NULL,	{NULL},	NULL,
		"Use: \"!writeto fn frame\"; \twrites the frame to the file fn.",
		&frcom_write,
		2, { FCPAR_STRING, FCPAR_LISTFR },
		FCRET_PRINTED
	},{
		"writetree",	NULL,	{ "(T)", NULL},	NULL,
		"Use: \"!writetree frame(s)\"; \twrites the frame subtree to a file (with matrices).",
		&frcom_writetree,
		1, { FCPAR_LISTFR },
		FCRET_PRINTED
	},{
		"writetreeto",	NULL,	{ NULL, "(T)", NULL},	NULL,
		"Use: \"!writetreeto fn frame\"; \twrites the frame subtree to the file fn.",
		&frcom_writetree,
		2, { FCPAR_STRING, FCPAR_LISTFR },
		FCRET_PRINTED
	},{
		"read",	NULL,	{ "noname", NULL},	NULL,
		"Use: \"!read fname >dest\"; \treads the given file fname to a new subtree.",
		&frcom_read,
		1, { FCPAR_STRING },
		FCRET_ONE
	},{
		"append",	NULL,	{NULL},	NULL,
		"Use: \"!append frame input\"; \tappends the given input to the frame (experimental!!!).",
		&frcom_append,
		2, { FCPAR_LISTFR, FCPAR_STRING },
		FCRET_PRINTED
	},{
	



		/*********	Simple matrix manipulation	*********/
		/********************************************************/
	
	
		"move",		NULL,	{NULL},	"()",
		"Use: \"!move src-list [>dest-list]\"; \tmoves (copies, deletes) frames within the tree.",
		&frcom_move,
		1, { FCPAR_LISTFR0 },
		FCRET_PARTLIST
	},{
		"mmove",	"%s",	{NULL},	"()",
		"Use: \"!mmove src-list [>dest-list]\"; \tmoves (copy, del) matrices-only within the tree.",
		&frcom_mmove,
		1, { FCPAR_LISTMX0 },
		FCRET_PARTLIST
	},{
		"flatten",	NULL,	{NULL},	"()",
		"Use: \"!flatten src-list >dest-list\"; \tflattens frame-subtrees within the tree.",
		&frcom_flatten,
		1, { FCPAR_LISTFR0 },
		FCRET_LIST
	},{
		"pivot",	"%s~p%s%s",	{NULL},	NULL,
		"Use: \"!pivot row col matrix\"; \tthe pivot is applied to one given matrix.",
		&frcom_pivot,
		3, { FCPAR_INTEGER, FCPAR_INTEGER, FCPAR_ONEMX },
		FCRET_REPLMX1
	},{
		"pivot-pr",	NULL,	{NULL},	NULL,
		"Use: \"!pivot-pr row col matrix\"; \tprinting the pivot applied to the matrix.",
		&frcom_pivot,
		3, { FCPAR_INTEGER, FCPAR_INTEGER, FCPAR_ONEMX },
		FCRET_PRINTED
	},{
		"dual",		"%s#",	{NULL},	NULL,
		"Use: \"!dual matrix(ces)\"; \ttranspose is applied to given matrix(ces).",
		&frcom_dual,
		1, { FCPAR_LISTMX },
		FCRET_REPLMXL
	},{
		"delete",	"%s~d%s",	{NULL},	NULL,
		"Use: \"!delete id matrix\"; \tthe line of given (id) is deleted from the matrix.",
		&frcom_delete,
		2, { FCPAR_INTEGER, FCPAR_ONEMX },
		FCRET_REPLMX1
	},{
		"contract",	"%s~c%s",	{NULL},	NULL,
		"Use: \"!contract id matrix\"; \tthe line of given (id) is contracted from the matrix.",
		&frcom_contract,
		2, { FCPAR_INTEGER, FCPAR_ONEMX },
		FCRET_REPLMX1
		
	},{
		"deleach",	"%s~de%d",	{"((T))", NULL},	"(((0t)))",
		"Use: \"!deleach mat(s) >out-list\"; \tmakes list by deleting each element from the matrix.",
		&frcom_deleteeach,
		1, { FCPAR_LISTMX },
		FCRET_LIST
	},{
		"coneach",	"%s~ce%d",	{"((T))", NULL},	"(((0t)))",
		"Use: \"!coneach mat(s) >out-list\"; \tmakes list by contracting each element from the matrix.",
		&frcom_contracteach,
		1, { FCPAR_LISTMX },
		FCRET_LIST
	},{
		"pfield",	NULL,	{"", NULL},	NULL,
		"Use: \"!pfield newpf\"; \tswitches (temporarily) to the given new pfield.",
		&frcom_pfield,
		1, { FCPAR_STRING },
		FCRET_PRINTED
	},{
		"import",	"%s<",	{"Id0", NULL},	NULL,
		"Use: \"!import transl_nm frame(s)\"; \timports the frames by translation transl_nm.",
		&frcom_import,
		2, { FCPAR_STRING, FCPAR_LISTFR },
		FCRET_REPLMXL
	},{
	
	


		/*********	Structural matrix tests	*********/
		/************************************************/
	
	
		"fan",		NULL,	{NULL},	NULL,
		"Use: \"!fan mat-list\"; \tprints the max fans in the matrices.",
		&frcom_hasfan,
		1, { FCPAR_LISTFRM },
		FCRET_PRINTED
	},{
		"hasfan",	NULL,	{"-1", NULL},	NULL,
		"Use: \"!hasfan f mat-list\"; \twhether there is a fan of len f in the matrices.",
		&frcom_hasfan,
		2, { FCPAR_INTEGER, FCPAR_LISTFRM },
		FCRET_PRINTED_F
	},{
		"bwidth3",	NULL,	{NULL},	NULL,
		"Use: \"!bwidth3 mat-list\"; \twhich of (3-conn!) matrices have branch-width 3.",
		&frcom_bwidth3test,
		1, { FCPAR_LISTFRM },
		FCRET_PRINTED_F
	},{
		"minor",	NULL,	{NULL},	NULL,
		"Use: \"!minor mat-list min-list\"; \twhich matrices have minor in min-list.",
		&frcom_minorlist,
		2, { FCPAR_LISTFRM, FCPAR_LISTMX },
		FCRET_PRINTED_F
	},{
		"tmajor",	NULL,	{NULL},	NULL,
		"Use: \"!tmajor mat-list min-list\"; \twhich matrices are tight majors of min-list.",
		&frcom_tmajorlist,
		2, { FCPAR_LISTFRM, FCPAR_LISTMX },
		FCRET_PRINTED_F
	},{
		"inpfield",	NULL,	{NULL},	NULL,
		"Use: \"!inpfield mat-list\"; \twhich of the matrices are properly represented.",
		&frcom_pfreprlist,
		1, { FCPAR_LISTFRM },
		FCRET_PRINTED_F
	},{
		"equiv",	NULL,	{NULL},	NULL,
		"Use: \"!equiv mat-list1 mat-list2\"; \twhich of list1 are (unlab) equiv to some in list2.",
		&frcom_equivalent,
		2, { FCPAR_LISTFRM, FCPAR_LISTMX },
		FCRET_PRINTED_F
	},{
		"unique",	NULL,	{NULL},	NULL,
		"Use: \"!unique mat-list alt-list\"; \tmake list unique using altern. repres. from alt-list.",
		&frcom_makeunique,
		2, { FCPAR_LISTFRM, FCPAR_LISTMX },
		FCRET_PRINTED_F
	},{
		"mhash",	NULL,	{"-1", NULL},	NULL,
		"Use: \"!mhash h-value mat-list\"; \twhich of list have matroid hash-value = h-value.",
		&frcom_matroidhash,
		2, { FCPAR_INTEGER, FCPAR_LISTFR },
		FCRET_PRINTED_F
	},{
	
	


		/*********	Matrix constructions	*********/
		/************************************************/
	
	
		"autom",	NULL,	{NULL},	NULL,
		"Use: \"!autom matrix\"; \tprints all automorphisms of the matrix.",
		&frcom_automorph,
		1, { FCPAR_ONEMX },
		FCRET_PRINTED
	},{
		"extend",	NULL,	{"b", "((T))", NULL},	"(((0t)))",
				/* name-pattern GEN_?EXTNAME for extensions is in gener.h */
		"Use: \"!extend [rcb]* matrix\"; \tgenerates (co)extensions of the matrix by \"rcb..\".",
		&frcom_extend,
		2, { FCPAR_STRING, FCPAR_LISTFRM },
		FCRET_LIST
	},{
		"extendsize",	NULL,	{NULL,NULL, "((T))", NULL},	"(((0t)))",
				/* name-pattern GEN_?EXTNAME for extensions is in gener.h */
		"Use: \"!extendsize r c matrix\"; \tgenerates (co)extensions of the matrix to size r x c.",
		&frcom_extend,
		3, { FCPAR_INTEGER, FCPAR_INTEGER, FCPAR_ONEFRM },
		FCRET_LIST
	},{




		/*********	Command-flow Control	*********/
		/************************************************/
	
	
		"restart",	NULL,	{NULL},	NULL,
		"Use: \"!restart\"; \trestart command processing in the whole tree.",
		&frcom_restart,
		0, { 0 },
		FCRET_PRINTED	/* FCRET_NOTHING ?? */
	},{
		"exit",	NULL,	{"0", NULL},	NULL,
		"Use: \"!exit num\"; \tstop processing of all commands immediately, exit with num.",
		&frcom_comexit,
		1, { FCPAR_INTEGER },
		FCRET_PRINTED	/* FCRET_NOTHING ?? */
	},{
		"skip",	NULL,	{"1", NULL},	NULL,
		"Use: \"!skip num\"; \tskip the next num commands in the current frame.",
		&frcom_comskip,
		1, { FCPAR_INTEGER },
		FCRET_PRINTED	/* FCRET_NOTHING ?? */
	},{
		"iflist",	NULL,	{NULL},	NULL,
		"Use: \"!iflist len [<=>!] fr-list\"; \twhether the frame-list has len?? frames.",
		&frcom_iflist,
		3, { FCPAR_INTEGER, FCPAR_STRING, FCPAR_LISTFR0 },
		FCRET_PRINTED	/* FCRET_NOTHING ?? */
	},{
	//************* what about !iffile testing existence of a file on disk ???
	
	
	
	
	
/*		
		"t",		NULL,	{NULL},	NULL,
		"Use: \"!\"; \tsample...",
		&frcom_test,
		1, { FCPAR_ONEFR },
		FCRET_LIST
*/		//FCRET_PRINTED
		
			/*****		*****/
	
//	},{

#define	FR_COMSEXTRA	1	/* (extra command handles included) */
#include "frcoms-more.inc"
#undef	FR_COMSEXTRA

		NULL,NULL,{NULL},NULL,NULL,NULL,0,{0},0	/* must end with NULL's! */
	}
};




































