
/********************************************************************************
 *										*
 *				 M A C E K					*
 *		("Matroids Also Computed Efficiently" Kit)			*
 *										*
 * A set of tools and routines for computations with representable matroids.	*
 * Copyright (C) 2001,2002  Petr Hlineny.					*
 *										*
 *   This program is free software; you can redistribute it and/or modify	*
 *   it under the terms of the GNU General Public License as published by	*
 *   the Free Software Foundation; either version 2 of the License, or		*
 *   (at your option) any later version.					*
 *   You should have received a copy of the GNU General Public License		*
 *   along with this program; if not, write to the Free Software		*
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.	*
 * 										*
 * See the top-level "ABOUT" and "doc/ *" files for more information.		*
 *										*
 ********************************************************************************/














/**
 * 
 * various general functions for the package.....
 * 
 * 
**/



#include "macek.h"










/******************	Starting and final tasks	*******************/
/**************************************************************************/


/**
 * 
 * the starting and final functions of the program.........
 * 
 * begin() is called before debug check, there is no pfield selected yet!
 * 
**/

int	macek_begin(void) {
	
	SRANDOM(time(NULL));
	errorout = stderr;
	debugout = printout = stdout;
	return 1;
}


void    ematrix_freeall(void) ;

int	macek_final(void) {
	
	final_check();
	
	if (errorout!=stderr && errorout!=stdout)  fclose(errorout);
	if (debugout!=stderr && debugout!=stdout)  fclose(debugout);
	if (printout!=stderr && printout!=stdout)  fclose(printout);
	
	struct_allbases_recfree();
	struct_automorphisms_recfree();
	ematrix_freeall();
	return 0;
}












/******************	Handling arbitrary lists	*******************/
/**************************************************************************/


#undef CURDLEV
#define CURDLEV	4


/**
 * A "list" is a structure of general type  **list_type, actually terminated by NULL.
 * (However, to avoid pointer compatability problems, we transfer lists also as *void...)
 * To allow lists to grow, we store additional information right after the terminating NULL:
 *  If ls[i]==NULL, then ls[i+1] is a pointer to the start of the list, and ls[i+2] (as long)
 *  is the length remaining after the end of the list (minus the terminating NULL and these two).
 * A special magic number is stored into one element just before the list starts.
 * Moreover, a list may be given pointing to an arbitrary internal element, and the code
 * here can recover the list's true start and end.
 * 
 * No internal element of a list may be set to NULL !!!
 * 
 * The internal function alist_getend() returns the end pointer (terminating NULL) of the
 * given list.
 * To speed-up this operation, we use a simple hash-table storing, for given list pointer
 * in alisttb_in[], the pointer to the true start of the list (if the list pointer is not
 * the true start, alisttb_mark[]==1), or the pointer to the terminating NULL of the list
 * (if the list pointer is the true start, alisttb_mark[]==2).
 * 
 * The internal function alist_setend() sets in the hash-table the end of the given list.
 * (If not the true start of the list is given, then it stores the true start first,
 * then the list end.)
 * If clr==1 is given, then it clears the apropriate entry from the table.
 * 
**/

#ifndef	ALISTHASHSZ
#define	ALISTHASHSZ	2507	/* the table of list ends, and its size */
#endif

#define	ALISTHASHP(l)	HASHPOINTER((l),ALISTHASHSZ)

static short	alisttb_mark[ALISTHASHSZ],
		alisttb_first = 1;
static void	**alisttb_in[ALISTHASHSZ],
		**alisttb_out[ALISTHASHSZ];


void**	alist_getend(void **ls) {
	void	**t, **x;
	int	i;
	
	if (alisttb_first) {	/* (first run of the function) */
		alisttb_first = 0;
		for (i=0; i<ALISTHASHSZ; i++)  alisttb_mark[i] = 0;
	}
	if (!ls)  return NULL;
	if (!*ls)  return ls;
#if ALISTHASHSZ>1
		/**
		 * Here we look into the hash-table, whether  ls  is at position i there.
		 * If alisttb_mark[i]==1 (stores the true start of the list), then we look
		 * into the table again to find the end of the list.
		 * If alisttb_mark[i]==2, we have got the end, which is then returned.
		**/
	i = ((unsigned)ls)%ALISTHASHSZ;
	if (alisttb_mark[i]>0 && alisttb_in[i]==ls) {
		if (alisttb_mark[i]==1) {
			t = alisttb_out[i];
			i = ((unsigned)t)%ALISTHASHSZ;
			if (alisttb_in[i]!=t)  i = -1;
		}
		if (i>=0)  if (alisttb_mark[i]==2) {
			if (*(alisttb_out[i])!=NULL)
				{PROGERROREXIT("Fatal error - probably broken list %p !",ls);}
			return alisttb_out[i];
		}
	}
#endif
		/**
		 * This is the default simple (but slow) catch-up - find the terminating
		 * NULL of the list, and return the pointer (plus some checks).
		**/
	for (x=ls; *x; x++) ;
	t = x[1];
	if (t[-1]!=(void*)12345678l)  {PROGERROREXIT("Fatal error - probably broken list %p: %p !",ls,t[-1]);}
	return x;
}


void	alist_setend(void **ls, void **lse, int clr) {
	long    *x, y;
	void    **pt;
	int	i;
	
	if (alisttb_first)  alist_getend(NULL);	/* (first run of the function) */
	pt = lse[1];
	x = (long*)(lse+2);  y = ls-pt;
	if (lse[0]!=NULL || *x<0 || *x>1000000 || y<0 || y>1000000 || pt[-1]!=(void*)12345678l)
		{PROGERROREXIT("Fatal error - probably broken list %p (%p, %p, %ld, %ld)!",ls,ls[0],pt[-1],*x,y);}
	
		/**
		 * If the given ls is not the true start of the list (as found at the end lse),
		 * then we must first store a pointer to the true start, and mark 1.
		**/
	if (!clr && ls!=lse[1]) {
		i = ((unsigned)ls)%ALISTHASHSZ;
		alisttb_mark[i] = 1;
		alisttb_in[i] = ls;
		alisttb_out[i] = lse[1];
	}
		/**
		 * If clr is given, then we clear the table entry lse[1] if it exists.
		 * Otherwise, we store the pointer to the end of the list, and mark 2.
		**/
	i = ((unsigned)(lse[1]))%ALISTHASHSZ;
	if (clr) {
		if (alisttb_in[i]==lse[1])
			alisttb_mark[i] = 0;
	} else {
		alisttb_mark[i] = 2;
		alisttb_in[i] = lse[1];
		alisttb_out[i] = lse;
	}
}



/**
 * Here we create and dispose lists - these functions are for general use.
 * - new_alist() creates a new empty list of the given initial size.
 * - alist_free() frees the given list (possible to give any element as a pointer), but NOT the
 *  list elements.
 * - dispose_alist_ext() frees all the given list elements (using an optional disposing function
 *  given in *ff), and then frees the list itself.
 * 
**/

void*	new_alist(long sz) {
	void	**pt;
	long	*x;
	
	pt = MMALLOC((sz+4)*sizeof(pt[0]));  pt += 1;
	pt[-1] = (void*)12345678l;	/* the first allocated element is used for a magic number !!! */
	pt[0] = NULL;  pt[1] = pt;
	x = (long*)(pt+2);  x[0] = sz;
	alist_setend(pt,pt,0);
	return pt;
}

void	alist_free(void *ls) {
	void	**et,**st;
	
	et = alist_getend(ls);  st = et[1];
	if (st[-1]!=(void*)12345678l)  {PROGERROREXIT("Fatal error - probably broken list %p: %p !",ls,st[-1]);}
	alist_setend(st,et,1);	/* (to clear the hash-table entry of ls, also consistency check) */
	FREE(st-1);
}


void	dispose_alist_ext(void *ls, void (*ff)(void*)) {
	void	**pt,**et,**x;
	
	if (!ls)  {PROGERROR("Cannot dispose NULL list!"); return;}
	et = alist_getend(ls);  pt = et[1];
	
	for (x=pt; *x; x++) {	/* frees all elements of the list, possibly using supplied (*ff)() */
		if (ff)  (*ff)(*x);
		else  FREE(*x);
	}
	if (x!=et)  {PROGERROREXIT("Fatal error - probably broken list %p: %p %p !",ls,x,et);}
	alist_free(et);
}
void	dispose_alist(void *ls) {
	dispose_alist_ext(ls,NULL);
}


void*	alist_getstart(void *ls) {
	void	**et;
	
	if (!ls)  return NULL;
	et = alist_getend(ls);
	return et[1];
}

int	alist_getlength(void *ls) {
	void	**et;
	
	if (!ls)  return 0;
	et = alist_getend(ls);
	return et-(void**)et[1];
}


/**
 * This function appends a new element ap to the end of the given list.
 * (Again, it is possible to give any element of the list as a pointer ls.)
 * If the list is not sufficiently large, it is reallocated to twice its size.
 * The list is returned back (possibly different address, so it must be stored!).
**/

void*	alist_append(void *ls, void *ap) {
	void	**pt,**et;
	long	df, l,x;
	
	if (!ap)  {PROGERROR("Cannot append NULL element to a list!"); return ls;}
	if (!ls)  ls = new_alist(10);
	et = alist_getend(ls);
	pt = et[1];
	l = *((long*)(et+2));
		/**
		 * If there is no more space in the list, we must re-allocate it, and
		 * set the new values of pt,et and ls.
		 * Also, the table entry of the old list must be cleared.
		**/
	if (l==0) {
		alist_setend(ls,et,1);
		df = (void**)ls-pt;
		x = (et-pt);  l = x+2;
		pt = MREALLOC(pt-1,(x+4+l)*sizeof(pt[0]));
		pt += 1;	/* (the first allocated element is used for a magic number !!!) */
		et = pt+x;  ls = pt+df;
	}
		/**
		 * We store the given element ap at the end of the current list,
		 * and move the additional information one-up, decrementing the available length.
		 * Finally, we record the new list end in the table.
		**/
	et[0] = ap;  et[1] = NULL;
	et[2] = pt;
	*((long*)(et+3)) = l-1;
	alist_setend(ls,et+1,0);
	return ls;
}

void*	alist_applist(void *ls, void *apl) {
	void	**x;
	
	if (!ls)  return apl;
	for (x=apl; x?*x:0; x++)  ls = alist_append(ls,*x);
	if (apl)  alist_free(apl);
	return ls;
}


/**
 * This function deletes one element pointed by ls from the list (the element itself is not freed).
 * The remaining elements in the list are simply shifted by one to the front.
 * The list is returned back (address is unchanged).
**/

void*	alist_delete(void *ls) {
	void	**et;
	
	if (!ls)  {PROGERROR("Cannot delete NULL element from a list!"); return ls;}
	for (et=ls; *et; et++)  et[0] = et[1];
	et[0] = et[1];
	*((long*)(et+1)) = *((long*)(et+2))+1;
	alist_setend(ls,et-1,0);
	return ls;
}


/**
 * This function inserts one element to the position pointed by ins in the list.
 * The list is possibly enlarged like in alist_append(), and the remaining elements
 * are shifted by one to the end.
 * The list is returned back (possibly different address, so it must be stored!).
**/

void*	alist_insert(void *ls, void *ins, void *ai) {
	void	**et;
	long	di;
	
	if (!ins || !ai)  {PROGERROR("Cannot insert NULL element or NULL position to a list!"); return ls;}
	if (!ls)  ls = alist_getstart(ins);
	di = (void**)ins-(void**)ls;	/* (must keep the difference, since ls may change next) */
	DEBUG(CURDLEV+3,">> ls=%p,  ins=%p,  di=%ld\n",ls,ins,di);
	ls = alist_append(ls,ai);
	et = alist_getend(ls);  et--;
	if (di<0 || di>et-(void**)ls)  {PROGERROR("The insert position %p (%ld) is not in the list %p-%p?!",ins,di,ls,et);}
	for ( ; et-(void**)ls>di; et--)  et[0] = et[-1];
	et[0] = ai;	/* the inserting position is now prepared */
	return ls;
}


/**
 * This function returns a copy of the given list.
 * (The list ls may be given by an arbitrary internal position.)
**/

void*	alist_copy(void *ls) {
	void	**x, **st,**ln;
	
	if (!ls)  {PROGERROR("Cannot copy NULL list!"); return ls;}
	ln = new_alist(10);
	st = alist_getstart(ls);
	for (x=st; *x; x++)  ln = alist_append(ln,*x);
	return  ln+ ((void**)ls-st);
}





















/******************	File utilities	*******************/
/**********************************************************/



/**
 * 
 * 
 * trying to open fn in mode mod, using path and optional extension.......
 * 	(extension is forced first when writing and no ext...),
 * 	about overwrite-protection (not for /tmp),
 * 	creating missing directories on write...
 * 
**/

char	*fopen_dirused;		/* records the directory found in the path during open */

FILE*	fopen_path(char *fn, char *mod, char **path, char *dir, char *ext) {
	char	*dfn, *buf, *x;
	FILE	*fp = NULL;
	int	i,j,l,w, bl1,bl2;
	
	if (fn==NULL || path==NULL) {
		PROGERROR("No filename or path given for reading!");  return NULL;
	}
				/* removing leading and trailing whitespaces */
	while (*fn==' ' || *fn=='\t' || *fn=='\n')  fn++;
	dfn = MSTRDUP(fn);
	for (i=strlen(dfn)-1; i>=0 && (dfn[i]==' ' || dfn[i]=='\t' || dfn[i]=='\n'); i--) ;
	dfn[i+1] = 0;
	bl1 = strlen(dfn)+(ext?strlen(ext):0)+(dir?strlen(dir):0)+32;
	bl2 = strlen(path[0])+500;
	buf = MMALLOC((bl1+bl2)*sizeof(buf[0]));
	
	w = (mod[0]=='w');	/* the extension is preferred when writing and no extension exists in fn */
	for (x=dfn+strlen(dfn)-1; x>dfn && x[0]!='/' && x[0]!='.'; x--) ;
	if (w && x[0]=='.')  ext = NULL;
	
	i = -1;			/* looking for the file to open: */
	if (dfn[0]=='/' ||\
			(dfn[0]=='.'&&dfn[1]=='/') || (dfn[0]=='.'&&dfn[1]=='.'&&dfn[2]=='/'))
		for (j=0; j<2 && !fp; j++) {	/* (openning absolute file name) */
			strcpy(buf,dfn);
			if (j==1 && !ext)  continue;
			if (j==(!w) && ext) { strcat(buf,ext); }
			fp = fopen_dir(buf,mod,1);
	
	} else					/* (searching directory path) */
		for (i=0; !fp && path[i]; i++) for (j=0; j<4 && !fp; j++) {
			strncpy(buf,path[i],bl2-10);
			if (j%2==1 && !dir)  continue;
			if (j%2==1) { strcat(buf,"/");  strcat(buf,dir); }
			l = strlen(buf);
			strcat(buf,"/");  strcat(buf,dfn);
			if (j/2==1 && !ext)  continue;
			if (j/2==(!w) && ext) { strcat(buf,ext); }
			fp = fopen_dir(buf,mod,l+1);
	}
	if (i<0)  fopen_dirused = "";
	else  fopen_dirused = path[i-1];
	FREE(dfn);  FREE(buf);
	return fp;
}

FILE*   fopen_dir(char *fn, char *mod, int pre) {
	FILE    *fp;
	char    *path,*pp;
	extern int	file_overwrite;
	
	if (mod[0]!='w') {		/* normal open for reading */
		fp = fopen(fn,mod);
		return fp;
	}
	if (!file_overwrite) if	((fp = fopen(fn,"r"))!=NULL) {
		if (strncmp(fn,FR_AUTOWRITEPREF,strlen(FR_AUTOWRITEPREF))!=0)
			{USERERROR("Trying to overwrite an existing file \"%s\"!",fn);}
		fclose(fp);
	}
	
	path = MMALLOC(strlen(fn)+5);	/* special open for writing */
	strcpy(path,fn);  pp = path+pre;
	while ((pp = strchr(pp,'/'))!=NULL) {
		*pp = 0;		/* (creates all required subdirectories after pre) */
		mkdir(path,0755);
		*pp++ = '/';
	}
	fp = fopen(path,mod);
	FREE(path);
	return fp; 
}



/**
 * 
 * 
 * extracting the filename (not dir path and not extension)......... - a copy....
 * 
**/

char*	fname_extract(char *fn) {
	char	*x,*y,*z;
	
	for (x=y=z=fn; *x; x++) {
		if (*x=='/')  y = x+1;
		if (*x=='.')  z = x;
	}
	x = MSTRDUP(y);
	if (z>y)  x[(z-y)] = 0;
	return x;
}

char*	fdir_extract(char *fn) {
	char	*x,*y;
	
	x = MSTRDUP(fn);
	for (y=x; *x; x++) {
		if (*x=='/')  y = x;
	}
	y[0] = 0;
	return x;
}





		/**
		 * 
		 * 
		**/






































